/*******************************************************************************
 * Copyright (c) 2020 Aston University.
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the Eclipse
 * Public License, v. 2.0 are satisfied: GNU General Public License, version 3.
 *
 * SPDX-License-Identifier: EPL-2.0 OR GPL-3.0
 *
 * Contributors:
 *     Antonio Garcia-Dominguez - initial API and implementation
 ******************************************************************************/
package org.eclipse.hawk.timeaware.graph;

import org.eclipse.hawk.core.graph.IGraphNode;
import org.eclipse.hawk.epsilon.emc.EOLQueryEngine;
import org.eclipse.hawk.epsilon.emc.wrappers.GraphNodeWrapper;

/**
 * Variant of {@link GraphNodeWrapper} which keeps a strong reference, in case
 * we cannot be sure that we can simply retrieve the exact original node through
 * ID and optionally time. This is needed for the scoping wrappers in the
 * time-aware query DSL, as recreating those would essentially require
 * re-running the query itself.
 */
public class StrongGraphNodeWrapper extends GraphNodeWrapper {

	private IGraphNode node;

	public StrongGraphNodeWrapper(IGraphNode n, EOLQueryEngine containerModel) {
		super(n, containerModel);
		this.node = n;
	}

	@Override
	public IGraphNode getNode() {
		return node;
	}

}
